# -*- coding: utf-8 -*-

from dummy_spark.conf import SparkConf
from dummy_spark.context import SparkContext
from dummy_spark.rdd import RDD

__author__ = 'willmcginnis'

__all__ = [
    'RDD',
    'SparkConf',
    'SparkContext',
]
